@@ -61,10 +61,8 @@ function(x, fun, ..., usenames=FALSE, cores=1, filename="", overwrite=FALSE, wop
 	doclust <- FALSE
 	if (inherits(cores, "cluster")) {
 		doclust <- TRUE
-		ncores <- length(cores)
 	} else if (cores > 1) {
 		doclust <- TRUE
-		ncores <- cores
 		cores <- parallel::makeCluster(cores)
 		on.exit(parallel::stopCluster(cores), add=TRUE)
 	}
@@ -83,13 +81,15 @@ function(x, fun, ..., usenames=FALSE, cores=1, filename="", overwrite=FALSE, wop
 	expected <- test$nl * ncx
 
 	if (doclust) {
+		ncores <- length(cores)
+		export_args(cores, ..., caller="lapp")		
 		cfun <- function(i, ...)  do.call(fun, i, ...)
 		parallel::clusterExport(cores, "cfun", environment())
 		for (i in 1:b$n) {
 			v <- readValues(x, b$row[i], b$nrows[i], 1, ncx, dataframe=TRUE)
 			if (!usenames) colnames(v) <- NULL
 			v <- split(v, rep(1:ncores, each=ceiling(nrow(v) / ncores))[1:nrow(v)])
-			v <- unlist(parallel::parLapply(cores, v, cfun))
+			v <- unlist(parallel::parLapply(cores, v, cfun, ...))
 			if (length(v) != (expected * b$nrows[i])) {
 				out <- writeStop(out)
 				error("lapp", "output length of fun is not correct")
@@ -183,7 +183,7 @@ function(x, fun, ..., usenames=FALSE, recycle=FALSE, filename="", overwrite=FALS
 	}
 	nltot <- sum(nlyr(x)) + nlyr(out)
 	fact <- max(4, 4 * nltot / nlyr(out))
-	b <- writeStart(out, filename, overwrite, sources=sources(x), wopt=wopt, n=fact)
+	b <- writeStart(out, filename, overwrite, sources=unlist(sources(x)), wopt=wopt, n=fact)
 
 	for (i in 1:b$n) {
 		v <- lapply(1:length(x), function(s) readValues(x[s], b$row[i], b$nrows[i], 1, ncx, mat=TRUE))
