@@ -4,11 +4,30 @@ function(x, mx=10000, ...) {
 		if(prod(dim(x)) > mx) {
 			error("lines", "too many lines (you can increase the value of mx or use as.polygons)")
 		}
-		v <- as.polygons(x, dissolve=FALSE, values=FALSE)
+		v <- as.polygons(x[[1]], dissolve=FALSE, values=FALSE)
 		lines(v, ...)
 	}
 )
 
+
+setMethod("points", signature(x="SpatRaster"),
+function(x, ...) {
+		p <- as.points(x[[1]])
+		points(p, ...)
+	}
+)
+
+setMethod("polys", signature(x="SpatRaster"),
+function(x, mx=10000, dissolve=TRUE, ...) {
+		if(prod(dim(x)) > mx) {
+			error("lines", "too many lines (you can increase the value of mx or use as.polygons)")
+		}
+		p <- as.polygons(x[[1]], dissolve=dissolve)
+		polys(p, ...)
+	}
+)
+
+
 setMethod("lines", signature(x="SpatVector"),
 	function(x, y=NULL, col, lwd=1, lty=1, arrows=FALSE, alpha=1, ...)  {
 		n <- nrow(x)
@@ -112,3 +131,4 @@ setMethod("polys", signature(x="SpatVector"),
 	}
 )
 
+		
\ No newline at end of file
