@@ -1,18 +1,25 @@
 
 setMethod("makeTiles", signature(x="SpatRaster"),
 	function(x, y, filename="tile_.tif", extend=FALSE, na.rm=FALSE, ...) {
-		filename = trimws(filename[1])
+		filename <- trimws(filename[1])
 		filename <- filename[!is.na(filename)]
 		if (filename == "") error("makeTiles", "filename cannot be empty")
-		if (!inherits(y, "SpatRaster")) error("makeTiles", "y must be a SpatRaster")
 		opt <- spatOptions(filename="", ...)
-		ff <- x@ptr$make_tiles(y@ptr, extend[1], na.rm[1], filename, opt)
+		if (inherits(y, "SpatRaster")) {
+			ff <- x@ptr$make_tiles(y@ptr, extend[1], na.rm[1], filename, opt)
+		} else if (inherits(y, "SpatVector")) {
+			ff <- x@ptr$make_tiles_vect(y@ptr, extend[1], na.rm[1], filename, opt)		
+		} else {
+			error("makeTiles", "y must be a SpatRaster or SpatVector")
+		}
 		messages(x, "makeTiles")
-		return (ff)
+		ff
 	}
 )
 
 
+
+
 #		if (!hasValues(x)) error("makeTiles", "x has no values")
 #		y <- rast(y)[[1]]
 #		if (expand) y <- expand(y, ext(x), snap="out")
