@@ -152,7 +152,7 @@ void power(std::vector<double>& a, const std::vector<double>& b) {
 bool smooth_operator(std::string oper, bool &logical) {
 	std::vector<std::string> f {"==", "!=", ">", "<", ">=", "<="};
 	logical = std::find(f.begin(), f.end(), oper) != f.end();
-	f = {"+", "-", "*", "^", "/", "%"};
+	f = {"+", "-", "*", "^", "/", "%", "%/%"};
 	return (logical || (std::find(f.begin(), f.end(), oper) != f.end()));
 }
 
@@ -231,6 +231,10 @@ SpatRaster SpatRaster::arith(SpatRaster x, std::string oper, SpatOptions &opt) {
 			power(a, b);
 		} else if (oper == "%") {
 			 a % b;
+		} else if (oper == "%/%") {
+			for (size_t i=0; i<a.size(); i++) {
+				a[i] = trunc(a[i] / b[1]);
+			}
 		} else if (oper == "==") {
 			a == b;
 		} else if (oper == "!=") {
@@ -334,6 +338,12 @@ SpatRaster SpatRaster::arith(double x, std::string oper, bool reverse, SpatOptio
 					a[i] = std::fmod(a[i], x);
 				}
 			}
+		} else if (oper == "%/%") {
+			if (reverse) {
+				for(double& d : a) d = trunc(x / d);
+			} else {
+				for(double& d : a) d = trunc(d / x);
+			}
 		} else if (oper == "==") {
 			for(double& d : a) if (!std::isnan(d)) d = d == x;
 		} else if (oper == "!=") {
@@ -1241,7 +1251,7 @@ SpatRaster SpatRaster::isnan(SpatOptions &opt) {
 }
 
 
-SpatRaster SpatRaster::anynan(SpatOptions &opt) {
+SpatRaster SpatRaster::anynan(bool setnan, SpatOptions &opt) {
 
 	SpatRaster out = geometry(1);
 	out.setValueType(3);
@@ -1256,13 +1266,14 @@ SpatRaster SpatRaster::anynan(SpatOptions &opt) {
 		readStop();
 		return out;
 	}
+	double inval = setnan ? NAN : 0;
 	size_t nl = nlyr();
 	size_t nc = ncol();
 	for (size_t i=0; i<out.bs.n; i++) {
 		std::vector<double> v, w;
 		readBlock(v, out.bs, i);
 		size_t off = out.bs.nrows[i] * nc;
-		w.resize(off);
+		w.resize(off, inval);
 		for (size_t j=0; j<off; j++) {
 			for (size_t k=0; k<nl; k++) {
 				size_t cell = j + k * off;
@@ -1280,7 +1291,7 @@ SpatRaster SpatRaster::anynan(SpatOptions &opt) {
 }
 
 
-SpatRaster SpatRaster::allnan(SpatOptions &opt) {
+SpatRaster SpatRaster::nonan(bool setnan, SpatOptions &opt) {
 
 	SpatRaster out = geometry(1);
 	out.setValueType(3);
@@ -1291,6 +1302,48 @@ SpatRaster SpatRaster::allnan(SpatOptions &opt) {
 		return(out);
 	}
 
+	if (!out.writeStart(opt, filenames())) {
+		readStop();
+		return out;
+	}
+	double inval = setnan ? NAN : 0;
+	size_t nl = nlyr();
+	size_t nc = ncol();
+	for (size_t i=0; i<out.bs.n; i++) {
+		std::vector<double> v, w;
+		readBlock(v, out.bs, i);
+		size_t off = out.bs.nrows[i] * nc;
+		w.resize(off, 1);
+		for (size_t j=0; j<off; j++) {
+			for (size_t k=0; k<nl; k++) {
+				size_t cell = j + k * off;
+				if (std::isnan(v[cell])) {
+					w[j] = inval;
+					continue;
+				}
+			}
+		}
+		if (!out.writeBlock(w, i)) return out;
+	}
+	readStop();
+	out.writeStop();
+	return(out);
+}
+
+
+
+SpatRaster SpatRaster::allnan(bool setnan, SpatOptions &opt) {
+
+	SpatRaster out = geometry(1);
+	out.setValueType(3);
+
+    if (!hasValues()) return out;
+	if (!readStart()) {
+		out.setError(getError());
+		return(out);
+	}
+	double outval = setnan ? NAN : 0;
+
 	if (!out.writeStart(opt, filenames())) {
 		readStop();
 		return out;
@@ -1306,7 +1359,7 @@ SpatRaster SpatRaster::allnan(SpatOptions &opt) {
 			for (size_t k=0; k<nl; k++) {
 				size_t cell = j + k * off;
 				if (!std::isnan(v[cell])) {
-					w[j] = 0;
+					w[j] = outval;
 					continue;
 				}
 			}
