@@ -90,10 +90,12 @@ static void __errorHandler(const char *fmt, ...) {
 	char buf[BUFSIZ], *p;
 	va_list ap;
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	size_t n = BUFSIZ;
+	vsnprintf(buf, n, fmt, ap);
+//	vsprintf(buf, fmt, ap);
 	va_end(ap);
 	p = buf + strlen(buf) - 1;
-	if(strlen(buf) > 0 && *p == '\n') *p = '\0';
+	if (strlen(buf) > 0 && *p == '\n') *p = '\0';
     errNoCall(buf); 
 	return; 
 } 
@@ -102,10 +104,12 @@ static void __warningHandler(const char *fmt, ...) {
 	char buf[BUFSIZ], *p;
 	va_list ap;
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	size_t n = BUFSIZ;
+	vsnprintf(buf, n, fmt, ap);
+//	vsprintf(buf, fmt, ap);
 	va_end(ap);
 	p = buf + strlen(buf) - 1;
-	if(strlen(buf) > 0 && *p == '\n') *p = '\0';
+	if (strlen(buf) > 0 && *p == '\n') *p = '\0';
     warnNoCall(buf); 
 	return;
 }
@@ -117,7 +121,9 @@ static void __errorHandler(const char *fmt, ...) {
 	char buf[BUFSIZ], *p;
 	va_list ap;
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	size_t n = BUFSIZ;
+	vsnprintf(buf, n, fmt, ap);
+//	vsprintf(buf, fmt, ap);
 	va_end(ap);
 	p = buf + strlen(buf) - 1;
 	if(strlen(buf) > 0 && *p == '\n') *p = '\0';
@@ -129,7 +135,9 @@ static void __warningHandler(const char *fmt, ...) {
 	char buf[BUFSIZ], *p;
 	va_list ap;
 	va_start(ap, fmt);
-	vsprintf(buf, fmt, ap);
+	size_t n = BUFSIZ;
+	vsnprintf(buf, n, fmt, ap);
+//	vsprintf(buf, fmt, ap);
 	va_end(ap);
 	p = buf + strlen(buf) - 1;
 	if(strlen(buf) > 0 && *p == '\n') *p = '\0';
@@ -468,7 +476,16 @@ std::vector<unsigned> &gid, std::vector<unsigned> &gp, std::vector<unsigned> &ho
 	if (npts < 0) {
 		msg = "GEOS exception 9";
 		return false;
-	}
+	} 
+	if (npts == 0) { // for #813
+		x.push_back(NAN);
+		y.push_back(NAN);
+		gid.push_back(i);			
+		gp.push_back(j);			
+		hole.push_back(0);
+		return true;
+	}	
+		
 	double xvalue = 0;
 	double yvalue = 0;
 	for (int p=0; p < npts; p++) {
@@ -498,6 +515,16 @@ std::vector<unsigned> &gid, std::vector<unsigned> &gp, std::vector<unsigned> &ho
 		msg = "exception 99";
 		return false;
 	}
+
+	if (npts == 0) { // for #813
+		x.push_back(NAN);
+		y.push_back(NAN);
+		gid.push_back(i);			
+		gp.push_back(j);			
+		hole.push_back(0);
+		return true;
+	}	
+
 	double xvalue = 0;
 	double yvalue = 0;
 	for (int p=0; p < npts; p++) {
@@ -584,7 +611,7 @@ SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSContextHand
 					out.setError(msg);
 					return out;
 				}
-			}	
+			}
 			if (track_ids) pts_ids.push_back(ids[i]);
 			f++;
 		} else if (gt == "LineString" || gt == "MultiLineString") {
@@ -616,9 +643,6 @@ SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSContextHand
 
 		} else if (gt == "GeometryCollection") {
 
-			//Rcpp::Rcout << np << std::endl;
-
-
 			size_t kk = 0; // introduced for intersect
 			for(size_t j = 0; j<np; j++) {
 
@@ -697,9 +721,11 @@ SpatVectorCollection coll_from_geos(std::vector<GeomPtr> &geoms, GEOSContextHand
 		out.push_back(v);
 		//Rcpp::Rcout << "lns" << std::endl;
 	}
+
 	if (pt_x.size() > 0) {
 		SpatVector v;
 		v.setGeometry("points", pt_gid, pt_gp, pt_x, pt_y, pt_hole);
+
 		if (track_ids) v.df.add_column(pts_ids, "ids");
 		out.push_back(v);
 		//Rcpp::Rcout << "pts" << std::endl;
