@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2021  Robert J. Hijmans
+// Copyright (c) 2018-2022  Robert J. Hijmans
 //
 // This file is part of the "spat" library.
 //
@@ -34,7 +34,8 @@ SpatDataFrame SpatDataFrame::skeleton() {
 	out.iv = std::vector<std::vector<long>>(iv.size());
 	out.sv = std::vector<std::vector<std::string>>(sv.size());
 	out.bv = std::vector<std::vector<int8_t>>(bv.size());
-	out.tv = std::vector<std::vector<SpatTime_t>>(tv.size());
+	out.tv = std::vector<SpatTime_v>(tv.size());
+	out.fv = std::vector<SpatFactor>(fv.size());
 	return out;
 }
 
@@ -81,24 +82,33 @@ int8_t SpatDataFrame::getBvalue(unsigned i, unsigned j) {
 	return bv[j][i];
 }
 
-std::vector<SpatTime_t> SpatDataFrame::getT(unsigned i) {
+SpatTime_v SpatDataFrame::getT(unsigned i) {
 	unsigned j = iplace[i];
 	return tv[j];
 }
 
 SpatTime_t SpatDataFrame::getTvalue(unsigned i, unsigned j) {
 	j = iplace[j];
-	return tv[j][i];
+	return tv[j].x[i];
 }
 
+SpatFactor SpatDataFrame::getF(unsigned i) {
+	unsigned j = iplace[i];
+	return fv[j];
+}
+
+SpatFactor SpatDataFrame::getFvalue(unsigned i, unsigned j) {
+	j = iplace[j];
+	return fv[j].subset({i});
+}
 
 SpatDataFrame SpatDataFrame::subset_rows(unsigned i) {
-	std::vector<unsigned> r = { i }; 
+	std::vector<unsigned> r = { i };
 	return subset_rows(r);
 }
 
 
-SpatDataFrame SpatDataFrame::subset_rows(std::vector<unsigned> range) { 
+SpatDataFrame SpatDataFrame::subset_rows(std::vector<unsigned> range) {
 
 	SpatDataFrame out;
 	unsigned nr = nrow();
@@ -118,8 +128,9 @@ SpatDataFrame SpatDataFrame::subset_rows(std::vector<unsigned> range) {
 	out.sv.resize(sv.size());
 	out.bv.resize(bv.size());
 	out.tv.resize(tv.size());
+	out.fv.resize(fv.size());
 	out.reserve(range.size());
-	
+
 	for (size_t i=0; i < range.size(); i++) {
 		for (size_t j=0; j < dv.size(); j++) {
 			out.dv[j].push_back(dv[j][range[i]]);
@@ -133,32 +144,44 @@ SpatDataFrame SpatDataFrame::subset_rows(std::vector<unsigned> range) {
 		for (size_t j=0; j < bv.size(); j++) {
 			out.bv[j].push_back(bv[j][range[i]]);
 		}
+		for (size_t j=0; j < fv.size(); j++) {
+			out.fv[j].v.push_back(fv[j].v[range[i]]);
+		}
 		for (size_t j=0; j < tv.size(); j++) {
-			out.tv[j].push_back(tv[j][range[i]]);
+			out.tv[j].x.push_back(tv[j].x[range[i]]);
 		}
 	}
+	for (size_t j=0; j < fv.size(); j++) {
+		out.fv[j].labels = fv[j].labels;
+	}
+	for (size_t j=0; j < tv.size(); j++) {
+		out.tv[j].step = tv[j].step;
+		out.tv[j].zone = tv[j].zone;
+	}
+
 	return out;
 }
 
 
-SpatDataFrame SpatDataFrame::subset_rows(std::vector<long> range) { 
+SpatDataFrame SpatDataFrame::subset_rows(std::vector<long> range) {
 	std::vector<unsigned> r(range.begin(), range.end());
 	return subset_rows(r);
 }
 
 
 SpatDataFrame SpatDataFrame::subset_cols(unsigned i) {
-	std::vector<unsigned> c = { i }; 
+	std::vector<unsigned> c = { i };
 	return subset_cols(c);
 }
 
-SpatDataFrame SpatDataFrame::subset_cols(std::vector<unsigned> range) { 
+SpatDataFrame SpatDataFrame::subset_cols(std::vector<unsigned> range) {
 	SpatDataFrame out;
 	unsigned dcnt=0;
 	unsigned icnt=0;
 	unsigned scnt=0;
 	unsigned bcnt=0;
 	unsigned tcnt=0;
+	unsigned fcnt=0;
 	for (size_t i=0; i < range.size(); i++) {
 		if (range[i] < 0 || range[i] >= ncol()) {
 			out.setError("invalid column");
@@ -187,11 +210,16 @@ SpatDataFrame SpatDataFrame::subset_cols(std::vector<unsigned> range) {
 			out.iplace.push_back(bcnt);
 			out.itype.push_back(3);
 			bcnt++;
-		} else {
+		} else if (itype[j] == 4) {
 			out.tv.push_back(tv[p]);
 			out.iplace.push_back(tcnt);
 			out.itype.push_back(4);
 			tcnt++;
+		} else { //if (itype[j] == 5) {
+			out.fv.push_back(fv[p]);
+			out.iplace.push_back(fcnt);
+			out.itype.push_back(5);
+			fcnt++;
 		}
 	}
 	return out;
@@ -216,8 +244,10 @@ unsigned SpatDataFrame::nrow() {
 			n = sv[0].size();
 		} else if (itype[0] == 3) {
 			n = bv[0].size();
-		} else {
+		} else if (itype[0] == 4) {
 			n = tv[0].size();
+		} else { //if (itype[0] == 5) {
+			n = fv[0].size();
 		}
 	}
 	return n;
@@ -244,6 +274,9 @@ void SpatDataFrame::add_row() {
 	for (size_t i=0; i < tv.size(); i++) {
 		tv[i].push_back(timeNA);
 	}
+	for (size_t i=0; i < fv.size(); i++) {
+		fv[i].push_back(0);
+	}
 }
 
 
@@ -266,6 +299,9 @@ void SpatDataFrame::add_rows(size_t n) {
 	for (size_t i=0; i < tv.size(); i++) {
 		tv[i].resize(s, timeNA);
 	}
+	for (size_t i=0; i < fv.size(); i++) {
+		fv[i].resize(s, 0);
+	}
 }
 
 
@@ -285,6 +321,9 @@ void SpatDataFrame::reserve(unsigned n) {
 	for (size_t i=0; i<tv.size(); i++) {
 		tv[i].reserve(n);
 	}
+	for (size_t i=0; i<fv.size(); i++) {
+		fv[i].reserve(n);
+	}
 }
 
 void SpatDataFrame::resize_rows(unsigned n) {
@@ -305,6 +344,9 @@ void SpatDataFrame::resize_rows(unsigned n) {
 	for (size_t i=0; i<tv.size(); i++) {
 		tv[i].resize(n, timeNA);
 	}
+	for (size_t i=0; i<fv.size(); i++) {
+		fv[i].resize(n, 0);
+	}
 }
 
 void SpatDataFrame::remove_rows(std::vector<unsigned> r) {
@@ -328,7 +370,10 @@ void SpatDataFrame::remove_rows(std::vector<unsigned> r) {
 			bv[i].erase(bv[i].begin() +r[j]);
 		}
 		for (size_t i=0; i<tv.size(); i++) {
-			tv[i].erase(tv[i].begin() +r[j]);
+			tv[i].x.erase(tv[i].x.begin() +r[j]);
+		}
+		for (size_t i=0; i<fv.size(); i++) {
+			fv[i].v.erase(fv[i].v.begin() +r[j]);
 		}
 	}
 }
@@ -344,17 +389,6 @@ void SpatDataFrame::resize_cols(unsigned n) {
 	}
 }
 
-// use template instead
-bool SpatDataFrame::add_column(std::vector<double> x, std::string name) {
-	unsigned nr = nrow();
-	if ((nr != 0) & (nr != x.size())) return false; 
-	iplace.push_back(dv.size());
-	itype.push_back(0);
-	names.push_back(name);
-	dv.push_back(x);
-	return true;
-}
-
 
 bool SpatDataFrame::remove_column(int i) {
 
@@ -396,10 +430,21 @@ bool SpatDataFrame::remove_column(std::string field) {
 }
 
 
+bool SpatDataFrame::add_column(std::vector<double> x, std::string name) {
+	unsigned nr = nrow();
+	if ((nr != 0) & (nr != x.size())) return false;
+	iplace.push_back(dv.size());
+	itype.push_back(0);
+	names.push_back(name);
+	dv.push_back(x);
+	return true;
+}
+
+
 
 bool SpatDataFrame::add_column(std::vector<long> x, std::string name) {
 	unsigned nr = nrow();
-	if ((nr != 0) & (nr != x.size())) return false; 
+	if ((nr != 0) & (nr != x.size())) return false;
 	iplace.push_back(iv.size());
 	itype.push_back(1);
 	names.push_back(name);
@@ -415,7 +460,7 @@ bool SpatDataFrame::add_column(std::vector<int> x, std::string name) {
 
 bool SpatDataFrame::add_column(std::vector<std::string> x, std::string name) {
 	unsigned nr = nrow();
-	if ((nr != 0) & (nr != x.size())) return false; 
+	if ((nr != 0) & (nr != x.size())) return false;
 	iplace.push_back(sv.size());
 	itype.push_back(2);
 	names.push_back(name);
@@ -425,7 +470,7 @@ bool SpatDataFrame::add_column(std::vector<std::string> x, std::string name) {
 
 bool SpatDataFrame::add_column(std::vector<int8_t> x, std::string name) {
 	unsigned nr = nrow();
-	if ((nr != 0) & (nr != x.size())) return false; 
+	if ((nr != 0) & (nr != x.size())) return false;
 	iplace.push_back(bv.size());
 	itype.push_back(3);
 	names.push_back(name);
@@ -433,9 +478,10 @@ bool SpatDataFrame::add_column(std::vector<int8_t> x, std::string name) {
 	return true;
 }
 
+
 bool SpatDataFrame::add_column_bool(std::vector<int> x, std::string name) {
 	unsigned nr = nrow();
-	if ((nr != 0) & (nr != x.size())) return false; 
+	if ((nr != 0) & (nr != x.size())) return false;
 	iplace.push_back(bv.size());
 	itype.push_back(3);
 	names.push_back(name);
@@ -453,9 +499,24 @@ bool SpatDataFrame::add_column_bool(std::vector<int> x, std::string name) {
 }
 
 
-bool SpatDataFrame::add_column(std::vector<SpatTime_t> x, std::string name) {
+bool SpatDataFrame::add_column_time(std::vector<SpatTime_t> x, std::string name, std::string step="seconds", std::string zone="") {
 	unsigned nr = nrow();
-	if ((nr != 0) & (nr != x.size())) return false; 
+	if ((nr != 0) & (nr != x.size())) return false;
+	iplace.push_back(tv.size());
+	itype.push_back(4);
+	names.push_back(name);
+	SpatTime_v v;
+	v.x = x;
+	v.zone=zone;
+	v.step=step;
+	tv.push_back(v);
+	return true;
+}
+
+
+bool SpatDataFrame::add_column(SpatTime_v x, std::string name) {
+	unsigned nr = nrow();
+	if ((nr != 0) & (nr != x.size())) return false;
 	iplace.push_back(tv.size());
 	itype.push_back(4);
 	names.push_back(name);
@@ -463,6 +524,16 @@ bool SpatDataFrame::add_column(std::vector<SpatTime_t> x, std::string name) {
 	return true;
 }
 
+bool SpatDataFrame::add_column(SpatFactor x, std::string name) {
+	unsigned nr = nrow();
+	if ((nr != 0) & (nr != x.size())) return false;
+	iplace.push_back(fv.size());
+	itype.push_back(5);
+	names.push_back(name);
+	fv.push_back(x);
+	return true;
+}
+
 
 void SpatDataFrame::add_column(unsigned dtype, std::string name) {
 	unsigned nr = nrow();
@@ -483,11 +554,16 @@ void SpatDataFrame::add_column(unsigned dtype, std::string name) {
 		std::vector<int8_t> bins(nr, 2);
 		iplace.push_back(bv.size());
 		bv.push_back(bins);
-	} else {
+	} else if (dtype == 4) {
 		SpatTime_t timeNA = NA<SpatTime_t>::value;
-		std::vector<SpatTime_t> sins(nr, timeNA);
+		SpatTime_v tins;
+		tins.resize(nr, timeNA);
 		iplace.push_back(tv.size());
-		tv.push_back(sins);
+		tv.push_back(tins);
+	} else {
+		SpatFactor fins(nr, 0);
+		iplace.push_back(fv.size());
+		fv.push_back(fins);
 	}
 	itype.push_back(dtype);
 	names.push_back(name);
@@ -509,8 +585,11 @@ bool SpatDataFrame::cbind(SpatDataFrame &x) {
 		} else if (x.itype[i] == 3) {
 			std::vector<int8_t> d = x.getB(i);
 			if (!add_column(d, nms[i])) return false;
+		} else if (x.itype[i] == 3) {
+			SpatTime_v d = x.getT(i);
+			if (!add_column(d, nms[i])) return false;
 		} else {
-			std::vector<SpatTime_t> d = x.getT(i);
+			SpatFactor d = x.getF(i);
 			if (!add_column(d, nms[i])) return false;
 		}
 	}
@@ -532,47 +611,54 @@ bool SpatDataFrame::rbind(SpatDataFrame &x) {
 		if (j < 0) { // not in df
 			size_t b = x.iplace[i];
 			add_column(x.itype[i], x.names[i]);
-			if (x.itype[i] == 0) { 
+			if (x.itype[i] == 0) {
 				size_t a = dv.size()-1;
-				dv[a].insert(dv[a].begin()+nr1, 
+				dv[a].insert(dv[a].begin()+nr1,
 					x.dv[b].begin(), x.dv[b].end());
-			} else if (x.itype[i] == 1) { 
+			} else if (x.itype[i] == 1) {
 				size_t a = iv.size()-1;
-				iv[a].insert(iv[a].begin()+nr1, 
+				iv[a].insert(iv[a].begin()+nr1,
 					x.iv[b].begin(), x.iv[b].end());
-			} else if (x.itype[i] == 2) { 
+			} else if (x.itype[i] == 2) {
 				size_t a = sv.size()-1;
-				sv[a].insert(sv[a].begin()+nr1, 
+				sv[a].insert(sv[a].begin()+nr1,
 					x.sv[b].begin(), x.sv[b].end());
-			} else if (x.itype[i] == 3) { 
+			} else if (x.itype[i] == 3) {
 				size_t a = bv.size()-1;
-				bv[a].insert(bv[a].begin()+nr1, 
+				bv[a].insert(bv[a].begin()+nr1,
 					x.bv[b].begin(), x.bv[b].end());
-			} else { 
+			} else if (x.itype[i] == 4) {
 				size_t a = tv.size()-1;
-				tv[a].insert(tv[a].begin()+nr1, 
-					x.tv[b].begin(), x.tv[b].end());
-			} 
+				tv[a].x.insert(tv[a].x.begin()+nr1,
+					x.tv[b].x.begin(), x.tv[b].x.end());
+			} else {
+				size_t a = fv.size()-1;
+				fv[a].v.insert(fv[a].v.begin()+nr1,
+					x.fv[b].v.begin(), x.fv[b].v.end());
+			}
 		} else {
 			size_t a = iplace[j];
 			size_t b = x.iplace[i];
 			if (itype[j] == x.itype[i]) {
-				if (itype[j] == 0) { 
-					dv[a].insert(dv[a].begin()+nr1, 
+				if (itype[j] == 0) {
+					dv[a].insert(dv[a].begin()+nr1,
 					x.dv[b].begin(), x.dv[b].end());
-				} else if (itype[j] == 1) { 
-					iv[a].insert(iv[a].begin()+nr1, 
+				} else if (itype[j] == 1) {
+					iv[a].insert(iv[a].begin()+nr1,
 					x.iv[b].begin(), x.iv[b].end());
-				} else if (itype[j] == 2) { 
-					sv[a].insert(sv[a].begin()+nr1, 
+				} else if (itype[j] == 2) {
+					sv[a].insert(sv[a].begin()+nr1,
 					x.sv[b].begin(), x.sv[b].end());
-				} else if (itype[j] == 3) { 
-					bv[a].insert(bv[a].begin()+nr1, 
+				} else if (itype[j] == 3) {
+					bv[a].insert(bv[a].begin()+nr1,
 					x.bv[b].begin(), x.bv[b].end());
-				} else { 
-					tv[a].insert(tv[a].begin()+nr1, 
-					x.tv[b].begin(), x.tv[b].end());
-				} 
+				} else if (itype[j] == 4) {
+					tv[a].x.insert(tv[a].x.begin()+nr1,
+					x.tv[b].x.begin(), x.tv[b].x.end());
+				} else {
+					fv[a].v.insert(fv[a].v.begin()+nr1,
+					x.fv[b].v.begin(), x.fv[b].v.end());
+				}
 			} else {
 				if (itype[j] == 2) {
 					sv[a].resize(nr1);
@@ -591,15 +677,20 @@ bool SpatDataFrame::rbind(SpatDataFrame &x) {
 							} else if (x.bv[b][k] == 1) {
 								sv[a].push_back("FALSE");
 							} else {
-								sv[a].push_back("NAS");								
+								sv[a].push_back("NAS");
 							}
 						}
+					} else if (x.itype[i] == 4){
+						for (size_t k=0; k<nr2; k++) {
+							sv[a].push_back(std::to_string(x.tv[b].x[k]));
+						}
 					} else {
 						for (size_t k=0; k<nr2; k++) {
-							sv[a].push_back(std::to_string(x.tv[b][k]));
+							std::string s = x.fv[b].getLabel(k);
+							sv[a].push_back(s);
 						}
 					}
-				} else if (itype[j] == 0) { 
+				} else if (itype[j] == 0) {
 					dv[a].resize(nr1);
 					if (x.itype[i] == 1) {
 						for (size_t k=0; k<nr2; k++) {
@@ -613,52 +704,63 @@ bool SpatDataFrame::rbind(SpatDataFrame &x) {
 							if (x.iv[b][k] < 2) {
 								dv[a].push_back(x.bv[b][k]);
 							} else {
-								dv[a].push_back(NAN);								
+								dv[a].push_back(NAN);
 							}
 						}
+					} else if (x.itype[i] == 4) {
+						for (size_t k=0; k<nr2; k++) {
+							dv[a].push_back(x.tv[b].x[k]);
+						}
 					} else {
 						for (size_t k=0; k<nr2; k++) {
-							dv[a].push_back(x.tv[b][k]);
+							dv[a].push_back(x.fv[b].v[k]);
 						}
 					}
 
-				} else if (itype[j] == 1) { 
+				} else if (itype[j] == 1) {
 					iv[a].resize(nr1);
+					long longNA = NA<long>::value;
 					if (x.itype[i] == 0) {
-						long longNA = NA<long>::value;
 						for (size_t k=0; k<nr2; k++) {
 							if (std::isnan(x.dv[b][k])) {
-								iv[a].push_back(longNA);								
+								iv[a].push_back(longNA);
 							} else {
 								iv[a].push_back(x.dv[b][k]);
 							}
 						}
 					} else if (x.itype[i] == 3) {
-						long longNA = NA<long>::value;
 						for (size_t k=0; k<nr2; k++) {
 							if (x.bv[b][k] > 1) {
-								iv[a].push_back(longNA);								
+								iv[a].push_back(longNA);
 							} else {
 								iv[a].push_back(x.bv[b][k]);
 							}
 						}
+					} else if (x.itype[i] == 5) {
+						for (size_t k=0; k<nr2; k++) {
+							if (x.fv[b].v[k] == 0) {
+								iv[a].push_back(longNA);
+							} else {
+								iv[a].push_back(x.fv[b].v[k]);
+							}
+						}
 					} else {
-						iv[a].resize(nr1 + nr2);						
+						iv[a].resize(nr1 + nr2);
 					}
-				} else if (itype[j] == 3) {					
+				} else if (itype[j] == 3) {
 					if (x.itype[i] == 1) {
 						for (size_t k=0; k<nr2; k++) {
 							if (x.dv[b][k] == 0 || x.dv[b][k] == 1) {
-								bv[a].push_back(x.dv[b][k]);								
+								bv[a].push_back(x.dv[b][k]);
 							} else {
 								bv[a].push_back(2);
 							}
 						}
 					} else {
-						bv[a].resize(nr1 + nr2);						
+						bv[a].resize(nr1 + nr2);
 					}
 				} else if (itype[j] == 4) {
-					tv[a].resize(nr1 + nr2);						
+					tv[a].resize(nr1 + nr2);
 				}
 			}
 		}
@@ -686,10 +788,10 @@ void SpatDataFrame::set_names(std::vector<std::string> nms){
 
 
 std::vector<std::string> SpatDataFrame::get_datatypes() {
-	std::vector<std::string> types = {"double", "long", "string", "bool", "time"};
+	std::vector<std::string> types = {"double", "long", "string", "bool", "time", "factor"};
 	std::vector<std::string> stype(itype.size());
 	for (size_t i=0; i<itype.size(); i++) {
-		stype[i] = types[itype[i]]; 
+		stype[i] = types[itype[i]];
 	}
 	return stype;
 }
@@ -698,19 +800,19 @@ std::vector<std::string> SpatDataFrame::get_datatypes() {
 std::string SpatDataFrame::get_datatype(std::string field) {
 	int i = where_in_vector(field, get_names(), false);
 	if (i < 0) return "";
-	i = itype[i]; 
-	std::vector<std::string> types = {"double", "long", "string", "bool", "time"};
-	return types[i]; 
+	i = itype[i];
+	std::vector<std::string> types = {"double", "long", "string", "bool", "time", "factor"};
+	return types[i];
 }
 
 std::string SpatDataFrame::get_datatype(int field) {
 	if ((field < 0) || (field > (int)(ncol()-1))) return "";
-	std::vector<std::string> types = {"double", "long", "string", "bool", "time"};
-	return types[itype[field]]; 
+	std::vector<std::string> types = {"double", "long", "string", "bool", "time", "factor"};
+	return types[itype[field]];
 }
 
 bool SpatDataFrame::field_exists(std::string field) {
-	return is_in_vector(field, get_names()); 
+	return is_in_vector(field, get_names());
 }
 
 
@@ -719,8 +821,7 @@ int SpatDataFrame::get_fieldindex(std::string field) {
 }
 
 
-// only doing this for one column for now
-SpatDataFrame SpatDataFrame::unique(int col) {
+SpatDataFrame SpatDataFrame::unique_col(int col) {
 	SpatDataFrame out = subset_cols(col);
 
 	if (out.hasError()) return out;
@@ -743,8 +844,11 @@ SpatDataFrame SpatDataFrame::unique(int col) {
 		std::sort(out.bv[0].begin(), out.bv[0].end());
 		out.bv[0].erase(std::unique(out.bv[0].begin(), out.bv[0].end()), out.bv[0].end());
 	} else if (out.itype[0] == 4) {
-		std::sort(out.tv[0].begin(), out.tv[0].end());
-		out.tv[0].erase(std::unique(out.tv[0].begin(), out.tv[0].end()), out.tv[0].end());
+		std::sort(out.tv[0].x.begin(), out.tv[0].x.end());
+		out.tv[0].x.erase(std::unique(out.tv[0].x.begin(), out.tv[0].x.end()), out.tv[0].x.end());
+	} else { //if (out.itype[0] == 4) {
+		std::sort(out.fv[0].v.begin(), out.fv[0].v.end());
+		out.fv[0].v.erase(std::unique(out.fv[0].v.begin(), out.fv[0].v.end()), out.fv[0].v.end());
 	}
 	return out;
 }
@@ -752,7 +856,7 @@ SpatDataFrame SpatDataFrame::unique(int col) {
 
 std::vector<int> SpatDataFrame::getIndex(int col, SpatDataFrame &x) {
 	size_t nd = nrow();
-	x = unique(col);
+	x = unique_col(col);
 	size_t nu = x.nrow();
 	std::vector<int> idx(nd, -1);
 	size_t ccol = iplace[col];
@@ -798,7 +902,16 @@ std::vector<int> SpatDataFrame::getIndex(int col, SpatDataFrame &x) {
 	} else if (x.itype[0] == 4) {
 		for (size_t i=0; i<nd; i++) {
 			for (size_t j=0; j<nu; j++) {
-				if (tv[ccol][i] == x.tv[0][j]) {
+				if (tv[ccol].x[i] == x.tv[0].x[j]) {
+					idx[i] = j;
+					continue;
+				}
+			}
+		}
+	} else { //if (x.itype[0] == 5) {
+		for (size_t i=0; i<nd; i++) {
+			for (size_t j=0; j<nu; j++) {
+				if (fv[ccol].v[i] == x.fv[0].v[j]) {
 					idx[i] = j;
 					continue;
 				}
@@ -820,39 +933,48 @@ std::vector<double> SpatDataFrame::as_double(size_t v) {
 		return out;
 	}
 	size_t j = iplace[v];
+	size_t n = nrow();
 	if (itype[v] == 0) return dv[j];
-//	if (itype[v] == 1) {
-	out.reserve(nrow());
+
+	out.reserve(n);
 	if (itype[v]==1) {
-		long longNA = NA<long>::value;		
-		for (size_t i=0; i<nrow(); i++){
-			if (tv[j][i] == longNA) {
-				out[i] = NAN;	
+		long longNA = NA<long>::value;
+		for (size_t i=0; i<n; i++){
+			if (iv[j][i] == longNA) {
+				out.push_back(NAN);
 			} else {
-				out[i] = (double)iv[j][i];			
+				out.push_back((double)iv[j][i]);
 			}
-		}	
+		}
 	} else if (itype[v]==3) {
-		for (size_t i=0; i<nrow(); i++){
+		for (size_t i=0; i<n; i++){
 			if (bv[j][i] > 1) {
-				out[i] = NAN;
+				out.push_back(NAN);
 			} else {
-				out[i] = (double)bv[j][i];			
+				out.push_back((double)bv[j][i]);
 			}
 		}
 	} else if (itype[v]==4) {
 		SpatTime_t timeNA = NA<SpatTime_t>::value;
-		for (size_t i=0; i<nrow(); i++){
-			if (tv[j][i] == timeNA) {
-				out[i] = NAN;				
+		for (size_t i=0; i<n; i++){
+			if (tv[j].x[i] == timeNA) {
+				out.push_back(NAN);
 			} else {
-				out[i] = (double)tv[j][i];
+				out.push_back((double)tv[j].x[i]);
 			}
 		}
-	} 
+	} else if (itype[v]==5) {
+		for (size_t i=0; i<n; i++){
+			if (fv[j].v[i] == 0) {
+				out.push_back(NAN);
+			} else {
+				out.push_back((double)fv[j].v[i]);
+			}
+		}
+	}
 	return out;
-}	
-	
+}
+
 
 std::vector<long> SpatDataFrame::as_long(size_t v) {
 	std::vector<long> out;
@@ -868,11 +990,11 @@ std::vector<long> SpatDataFrame::as_long(size_t v) {
 	if (itype[v] == 1) return iv[j];
 //	if (itype[v] == 0) {
 	out.reserve(nrow());
-	long longNA = NA<long>::value;		
+	long longNA = NA<long>::value;
 	if (itype[v] == 0) {
 		for (size_t i=0; i<nrow(); i++){
 			if (std::isnan(dv[j][i])) {
-				out.push_back(longNA);			
+				out.push_back(longNA);
 			} else {
 				out.push_back( (long)dv[j][i] );
 			}
@@ -880,24 +1002,32 @@ std::vector<long> SpatDataFrame::as_long(size_t v) {
 	} else if (itype[v]==3) {
 		for (size_t i=0; i<nrow(); i++){
 			if (bv[j][i] > 1) {
-				out[i] = longNA;
+				out.push_back(longNA);
 			} else {
-				out[i] = (long) bv[j][i];			
+				out.push_back((long) bv[j][i]);
 			}
 		}
 	} else if (itype[v]==4) {
 		SpatTime_t timeNA = NA<SpatTime_t>::value;
 		for (size_t i=0; i<nrow(); i++){
-			if (tv[j][i] == timeNA) {
-				out[i] = longNA;				
+			if (tv[j].x[i] == timeNA) {
+				out.push_back(longNA);
 			} else {
-				out[i] = (long) tv[j][i];
+				out.push_back((long) tv[j].x[i]);
 			}
 		}
-	} 
+	} else if (itype[v] == 5) {
+		for (size_t i=0; i<nrow(); i++){
+			if (fv[j].v[i] == 0) {
+				out.push_back(longNA);
+			} else {
+				out.push_back( (long)fv[j].v[i] );
+			}
+		}
+	}
 	return out;
-}	
-	
+}
+
 
 std::vector<std::string> SpatDataFrame::as_string(size_t v) {
 	std::vector<std::string> out;
@@ -918,7 +1048,102 @@ std::vector<std::string> SpatDataFrame::as_string(size_t v) {
 		for (size_t i=0; i<nrow(); i++) {
 			out.push_back(std::to_string(iv[j][i]));
 		}
+	} else if (dt == "factor") {
+		out = fv[j].getLabels();
 	}
 	return out;
 }
 
+
+std::vector<std::string> SpatDataFrame::get_timesteps() {
+	std::vector<std::string> s(ncol(), "");
+	size_t cnt = 0;
+	for (size_t i=0; i<ncol(); i++) {
+		if (itype[i] == 4) {
+			s[i] = tv[cnt].step;
+			cnt++;
+		}
+	}
+	return s;
+}
+
+std::vector<std::string> SpatDataFrame::get_timezones() {
+	std::vector<std::string> s(ncol(), "");
+	size_t cnt = 0;
+	for (size_t i=0; i<ncol(); i++) {
+		if (itype[i] == 4) {
+			s[i] = tv[cnt].zone;
+			cnt++;
+		}
+	}
+	return s;
+}
+
+
+
+std::vector<std::vector<std::string>> SpatDataFrame::to_strings() {
+	std::vector<std::vector<std::string>> out(ncol());
+	if (nrow() == 0) return out;
+	for (size_t i= 0; i<ncol(); i++) {
+		out[i] = as_string(i);
+	}
+	return out;
+}
+
+std::vector<std::string> SpatDataFrame::one_string() {
+	std::vector<std::string> out;
+	size_t n = nrow();
+	if (n == 0) return out;
+	std::vector<std::vector<std::string>> ss = to_strings();
+	size_t m = ncol();
+	out.reserve(n);
+	for (size_t i= 0; i<n; i++) {
+		std::string s = ss[0][i];
+		for (size_t j= 0; j<m; j++) {
+			s += ss[j][i];
+		}
+		out.push_back(s);
+	}
+	return out;
+}
+
+SpatDataFrame SpatDataFrame::unique() {
+	std::vector<std::string> s = one_string();
+	std::vector<std::string> u = s;
+    std::sort(u.begin(), u.end());
+    u.erase(std::unique(u.begin(), u.end()), u.end());
+	size_t nu = u.size();
+	size_t ns = s.size();
+	if (nu == ns) {
+		return *this;
+	}
+	std::vector<unsigned> keep;
+	keep.reserve(nu);
+	for (size_t i=0; i<nu; i++) {
+		for (size_t j=0; j<ns; j++) {
+			if (s[j] == u[i]) {
+				keep.push_back(j);
+				break;
+			}
+		}
+	}
+	return subset_rows(keep);
+}
+
+
+size_t SpatDataFrame::strwidth(unsigned i) {
+	size_t m = 0;
+	if (i < iplace.size()) {
+		if (itype[i] == 2) {
+			unsigned j = iplace[i];
+			if (j < sv.size()) {
+				std::vector<std::string> s = sv[j];
+				for (i = 0; i<s.size(); i++) {
+					m = std::max(m, s[i].size());
+				}
+			}
+		}
+	}
+	return m;
+}
+
